@@ -22,6 +22,9 @@ class ParamData(Model):
     #: Default value for parameter
     default: Optional[str] = None
 
+    #: Disables a default cast caused by ``default_arg_cast``
+    disable_type_caster_default_cast: bool = False
+
     #: Force this to be an 'out' parameter
     #:
     #: .. seealso:: :ref:`autowrap_out_params`
@@ -302,13 +305,7 @@ class ClassData(Model):
     #: Set the python name of the class to this
     rename: Optional[str] = None
 
-    #: If the type was created as a shared_ptr (such as via std::make_shared)
-    #: then pybind11 needs to be informed of this.
-    #:
-    #: https://github.com/pybind/pybind11/issues/1215
-    #:
-    #: One way you can tell we messed this up is if there's a double-free
-    #: error and the stack trace involves a unique_ptr destructor
+    #: This is deprecated and has no effect
     shared_ptr: bool = True
 
     #: If specified, put the class in a sub.pack.age. Ignored
