@@ -11,13 +11,15 @@ import re
 from urllib.request import Request, urlopen
 from urllib.error import HTTPError
 from collections import defaultdict
-import toml
+import tomli
+import tomli_w
 from contextlib import suppress
 
 from .setup import Setup
 from .generator_data import MissingReporter
 from .command.util import get_build_temp_path
 
+from . import overrides
 from . import platforms
 
 
@@ -130,6 +132,10 @@ class HeaderScanner:
     def run(self, args):
         s = get_setup()
         for wrapper in s.wrappers + s.static_libs:
+            print(
+                f'[tool.robotpy-build.wrappers."{wrapper.package_name}".autogen_headers]'
+            )
+
             for incdir in wrapper.get_include_dirs():
                 files = list(
                     sorted(
@@ -138,24 +144,23 @@ class HeaderScanner:
                     )
                 )
 
-                print("generate = [")
                 lastdir = None
                 for f in files:
                     if "rpygen" not in f:
                         thisdir = dirname(f)
                         if lastdir is None:
                             if thisdir:
-                                print("    #", PurePosixPath(thisdir))
+                                print("#", PurePosixPath(thisdir))
                         elif lastdir != thisdir:
                             print()
                             if thisdir:
-                                print("    #", PurePosixPath(thisdir))
+                                print("#", PurePosixPath(thisdir))
                         lastdir = thisdir
 
                         base = splitext(basename(f))[0]
                         f = PurePosixPath(f)
-                        print(f'    {{ {base} = "{f}" }},')
-                print("]")
+                        print(f'{base} = "{f}"')
+                print()
 
 
 class ImportCreator:
@@ -222,16 +227,53 @@ class PlatformInfo:
             help="Displays platform-specific information",
             parents=[parent_parser],
         )
+        parser.add_argument("--list", action="store_true", default=False)
+        parser.add_argument("platform", nargs="?", default=None)
         return parser
 
     def run(self, args):
-        p = platforms.get_platform()
-        print("platform:")
-        pprint.pprint(p)
-        print()
 
-        print("override keys:")
-        pprint.pprint(platforms.get_platform_override_keys(p))
+        if args.list:
+            for name in platforms.get_platform_names():
+                print(name)
+        else:
+
+            p = platforms.get_platform(args.platform)
+            print("platform:")
+            pprint.pprint(p)
+            print()
+
+            print("override keys:")
+            pprint.pprint(platforms.get_platform_override_keys(p))
+
+
+class ShowOverrides:
+    @classmethod
+    def add_subparser(cls, parent_parser, subparsers):
+        parser = subparsers.add_parser(
+            "show-override",
+            help="Displays pyproject.toml after override processing",
+            parents=[parent_parser],
+        )
+        parser.add_argument("toml", nargs="?", default="pyproject.toml")
+        parser.add_argument(
+            "-p",
+            "--platform",
+            default=None,
+            help="Use robotpy-build platform-info --list for available platforms",
+        )
+        return parser
+
+    def run(self, args):
+
+        p = platforms.get_platform(args.platform)
+        override_keys = platforms.get_platform_override_keys(p)
+
+        with open(args.toml, "rb") as fp:
+            d = tomli.load(fp)
+
+        overrides.apply_overrides(d, override_keys)
+        print(tomli_w.dumps(d))
 
 
 class MavenParser:
@@ -275,8 +317,8 @@ class MavenParser:
             self.os_names.add(plat.os)
             self.arch_names.add(plat.arch)
 
-        with open(args.toml_link) as fp:
-            config = toml.load(fp)["tool"]["robotpy-build"]
+        with open(args.toml_link, "rb") as fp:
+            config = tomli.load(fp)["tool"]["robotpy-build"]
 
             wrappers = {}
             if "wrappers" in config:
@@ -414,6 +456,7 @@ def main():
         HeaderScanner,
         ImportCreator,
         PlatformInfo,
+        ShowOverrides,
         MavenParser,
     ):
         cls.add_subparser(parent_parser, subparsers).set_defaults(cls=cls)
@@ -432,3 +475,7 @@ def main():
         retval = 0
 
     exit(retval)
+
+
+if __name__ == "__main__":
+    main()
