@@ -17,7 +17,7 @@ from os.path import (
 import posixpath
 import shutil
 import toposort
-from typing import Dict, List, Optional, Set
+from typing import Any, Dict, List, Optional, Set
 import yaml
 
 from header2whatever.config import Config
@@ -91,7 +91,7 @@ class Wrapper:
         self._gen_includes = []
 
         self.extension = None
-        if self.cfg.sources or self.cfg.generate:
+        if self.cfg.sources or self.cfg.autogen_headers:
             define_macros = [("RPYBUILD_MODULE_NAME", extname)] + [
                 tuple(d.split(" ")) for d in self.platform.defines
             ]
@@ -114,9 +114,9 @@ class Wrapper:
             # Used if the maven download fails
             self.supported_platforms = setup.project.supported_platforms
 
-        if self.cfg.generate and not self.cfg.generation_data:
+        if self.cfg.autogen_headers and not self.cfg.generation_data:
             raise ValueError(
-                "generation_data must be specified when generate is specified"
+                "generation_data must be specified when autogen_headers/generate is specified"
             )
 
         # Setup an entry point (written during build_clib)
@@ -239,10 +239,14 @@ class Wrapper:
     def get_extra_objects(self) -> Optional[List[str]]:
         pass
 
-    def get_type_casters(self, casters: Dict):
-        for header, types in self.cfg.type_casters.items():
-            for typ in types:
-                casters[typ] = header
+    def get_type_casters_cfg(self, casters: Dict[str, Dict[str, Any]]) -> None:
+        for ccfg in self.cfg.type_casters:
+            cfg = {"hdr": ccfg.header}
+            if ccfg.default_arg_cast:
+                cfg["darg"] = True
+
+            for typ in ccfg.types:
+                casters[typ] = cfg
 
     def all_deps(self):
         if self._all_deps is None:
@@ -288,9 +292,15 @@ class Wrapper:
     def _all_casters(self):
         casters = {}
         for dep in self.all_deps():
-            dep.get_type_casters(casters)
-        self.pkgcfg.get_pkg("robotpy-build").get_type_casters(casters)
-        self.get_type_casters(casters)
+            dep.get_type_casters_cfg(casters)
+        self.pkgcfg.get_pkg("robotpy-build").get_type_casters_cfg(casters)
+        self.get_type_casters_cfg(casters)
+
+        # make each configuration unique
+        for k, v in list(casters.items()):
+            v = v.copy()
+            v["typename"] = k
+            casters[k] = v
 
         # add non-namespaced versions of all casters
         # -> in theory this could lead to a conflict, but
@@ -525,11 +535,22 @@ class Wrapper:
         pkgcfg = pkgcfg.replace("##EXTRAINCLUDES##", extraincludes)
 
         type_casters = {}
-        self.get_type_casters(type_casters)
+        self.get_type_casters_cfg(type_casters)
         if type_casters:
-            pkgcfg += (
-                f"\n\ndef get_type_casters(casters):\n"
-                f"    casters.update({repr(type_casters)})\n"
+            pkgcfg += "\n\n"
+            pkgcfg += inspect.cleandoc(
+                f"""
+
+            def get_type_casters_cfg(casters):
+                casters.update({repr(type_casters)})
+
+            def get_type_casters(casters):
+                t = {{}}
+                get_type_casters_cfg(t)
+                for k, v in t.items():
+                    if "hdr" in v:
+                        casters[k] = v["hdr"]
+            """
             )
 
         with open(fname, "w") as fp:
@@ -550,7 +571,7 @@ class Wrapper:
         self, cxx_gen_dir, missing_reporter: Optional[MissingReporter] = None
     ):
 
-        if not self.cfg.generate:
+        if not self.cfg.autogen_headers:
             return
 
         cxx_gen_dir = join(cxx_gen_dir, self.name)
@@ -607,75 +628,74 @@ class Wrapper:
 
         generation_search_path = [self.root] + self._all_includes(False)
 
-        for gen in self.cfg.generate:
-            for name, header in gen.items():
+        for name, header in self.cfg.autogen_headers.items():
 
-                header = normpath(header)
-                for path in generation_search_path:
-                    header_path = join(path, header)
-                    if exists(header_path):
-                        break
+            header = normpath(header)
+            for path in generation_search_path:
+                header_path = join(path, header)
+                if exists(header_path):
+                    break
+            else:
+                import pprint
+
+                pprint.pprint(generation_search_path)
+                raise ValueError("could not find " + header)
+
+            if report_only:
+                templates = []
+                class_templates = []
+            else:
+                cpp_dst = join(cxx_gen_dir, f"{name}.cpp")
+                self.extension.sources.append(cpp_dst)
+                classdeps_dst = join(cxx_gen_dir, f"{name}.json")
+                classdeps[name] = classdeps_dst
+
+                hpp_dst = join(
+                    hppoutdir,
+                    "{{ cls['namespace'] | replace(':', '_') }}__{{ cls['name'] }}.hpp",
+                )
+
+                templates = [
+                    {"src": cpp_tmpl, "dst": cpp_dst},
+                    {"src": classdeps_tmpl, "dst": classdeps_dst},
+                ]
+                class_templates = [{"src": hpp_tmpl, "dst": hpp_dst}]
+
+            if only_generate is not None and not only_generate.pop(name, False):
+                continue
+
+            if per_header:
+                data_fname = join(datapath, name + ".yml")
+                if not exists(data_fname):
+                    print("WARNING: could not find", data_fname)
+                    data = HooksDataYaml()
                 else:
-                    import pprint
+                    data = self._load_generation_data(data_fname)
 
-                    pprint.pprint(generation_search_path)
-                    raise ValueError("could not find " + header)
+            # for each thing, create a h2w configuration dictionary
+            cfgd = {
+                # generation code depends on this being just one header!
+                "headers": [header_path],
+                "templates": templates,
+                "class_templates": class_templates,
+                "preprocess": True,
+                "pp_retain_all_content": False,
+                "pp_include_paths": pp_includes,
+                "pp_defines": pp_defines,
+                "vars": {"mod_fn": name},
+            }
 
-                if report_only:
-                    templates = []
-                    class_templates = []
-                else:
-                    cpp_dst = join(cxx_gen_dir, f"{name}.cpp")
-                    self.extension.sources.append(cpp_dst)
-                    classdeps_dst = join(cxx_gen_dir, f"{name}.json")
-                    classdeps[name] = classdeps_dst
+            cfg = Config(cfgd)
+            cfg.validate()
+            cfg.root = self.incdir
 
-                    hpp_dst = join(
-                        hppoutdir,
-                        "{{ cls['namespace'] | replace(':', '_') }}__{{ cls['name'] }}.hpp",
-                    )
+            hooks = Hooks(data, casters, report_only)
+            try:
+                processor.process_config(cfg, data, hooks)
+            except Exception as e:
+                raise ValueError(f"processing {header}") from e
 
-                    templates = [
-                        {"src": cpp_tmpl, "dst": cpp_dst},
-                        {"src": classdeps_tmpl, "dst": classdeps_dst},
-                    ]
-                    class_templates = [{"src": hpp_tmpl, "dst": hpp_dst}]
-
-                if only_generate is not None and not only_generate.pop(name, False):
-                    continue
-
-                if per_header:
-                    data_fname = join(datapath, name + ".yml")
-                    if not exists(data_fname):
-                        print("WARNING: could not find", data_fname)
-                        data = HooksDataYaml()
-                    else:
-                        data = self._load_generation_data(data_fname)
-
-                # for each thing, create a h2w configuration dictionary
-                cfgd = {
-                    # generation code depends on this being just one header!
-                    "headers": [header_path],
-                    "templates": templates,
-                    "class_templates": class_templates,
-                    "preprocess": True,
-                    "pp_retain_all_content": False,
-                    "pp_include_paths": pp_includes,
-                    "pp_defines": pp_defines,
-                    "vars": {"mod_fn": name},
-                }
-
-                cfg = Config(cfgd)
-                cfg.validate()
-                cfg.root = self.incdir
-
-                hooks = Hooks(data, casters, report_only)
-                try:
-                    processor.process_config(cfg, data, hooks)
-                except Exception as e:
-                    raise ValueError(f"processing {header}") from e
-
-                hooks.report_missing(data_fname, missing_reporter)
+            hooks.report_missing(data_fname, missing_reporter)
 
         if only_generate:
             unused = ", ".join(sorted(only_generate))
